/*
 * File : dlDiscPending.C
 *
 * Description : This file contains the class definition of the
 * dlDiscPending class.
 */

#include "types.h"
#include "frame.h"
#include "dlEvents.h"
#include "dllayer.h"
#include "dlDiscPending.h"
#include "dlDisconnected.h"

/*
 * Function : dlDiscPending()
 * 
 * Description : This is the constructor. It calls the
 * base class constructor.
 */

dlDiscPending::dlDiscPending(dllayer *dl) : dlStateMachine(dl) 
{
}

/*
 * Function : PrintState()
 * 
 * Description : Prints out the state. 
 */


void 
dlDiscPending::PrintState() 
{
	TRACE(dlSmModule,1,("State dlDiscPending\n"));
}

/*
 * Function : HandleSabm()
 * 
 * Description : Processes a SABM that has been received. 
 *	This is a collision situation. A DM is transmitted
 *	and state changed to dlDisconnected.
 */

dlStateMachine *
dlDiscPending::HandleSabm(int &errorCode) 
{
	TRACE(dlSmModule,1,("In HandleSabm\n"));
	// Collision of different un-numbered commands
	// Transmit a DM
	_dl->XmitFrame(errorCode,FALSE,DM,_dl->_lastFrameRecd->GetPf());
	CHANGE_STATE(dlDisconnected);
}

/*
 * Function : HandleDisc()
 * 
 * Description : Processes a DISC that has been received. 
 * This is a collision situation. A UA is transmitted,
 * and state changed to dlDisconnected.
 */

dlStateMachine *
dlDiscPending::HandleDisc(int &errorCode) 
{
	TRACE(dlSmModule,1,("In HandleDisc\n"));
	// Collision of similar un-numbered commands.
	// Transmit a UA
	_dl->XmitFrame(errorCode,FALSE,UA,_dl->_lastFrameRecd->GetPf());
	CHANGE_STATE(dlDisconnected);
}

/*
 * Function : HandleDm()
 * 
 * Description : Processes a DM that has been received.
 *	Ignore a mode-setting command (i.e. DM f=0 ). Else,
 * treat this as an acknowledgement of the DISC command
 *	that has been sent before, and transition to 
 *	dlDisconnected.
 */

dlStateMachine *
dlDiscPending::HandleDm(int &errorCode)
{
	errorCode = codeSuccessful;

	// Ignore a mode setting command, i.e., DM F=0
	if ( _dl->_lastFrameRecd->GetPf() == 0 )
		return this;
	
	CHANGE_STATE(dlDisconnected);
}
/*
 * Function : HandleUa()
 *
 * Description : Processes a UA that has been received.
 *	We are expecting a UA with F=1. Change state to
 * 	dlDisconnected if UA(1) is received. 
 */
 
dlStateMachine *
dlDiscPending::HandleUa(int &errorCode)
{
	errorCode = codeSuccessful;
	// Ignore UA with F=0!
	if ( _dl->_lastFrameRecd->GetPf() == 0 )
		return this;

	CHANGE_STATE(dlDisconnected);
}

/*
 * Function : HandleT1Timer()
 *
 * Description : This function handles the case of timer T1
 *	expiring. Retransmit the DISC command when T1 expires.
 * 	If the retry count has already reached a system limit
 *	of N2, notify the network layer, and change state to
 *	dlDisconnected. 
 */
 

dlStateMachine *
dlDiscPending::HandleT1Timer(int &errorCode)
{
	TRACE(dlSmModule,1,("dlDiscPending: In HandleT1Timer\n"));
	if ( _dl->_t1RetryCount < N2 ) {
		// We can retransmit the DISC command, p =1

		_dl->XmitFrame(errorCode,TRUE,DISC,1);
		if ( errorCode == codeSuccessful ) {		
			_dl->StartT1Timer(TRUE);					
			return this;
		}
		// Transmission failed.
		TRACE(dlSmModule,1,("Failed to transmit SABM!\n"));
		_dl->_nPlus1->Command(DL_DisconnectInd,errorCode, noPeerResponse);
		CHANGE_STATE(dlDisconnected);
	}
	else { 
		// already retransmitted N2 times, and still no response.
		TRACE(dlSmModule,1,("Timer T1 timed out N2 times\n"));
		_dl->_nPlus1->Command(DL_DisconnectInd,errorCode,noPeerResponse);
		CHANGE_STATE(dlDisconnected);
	}
}
